@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2012 Olaf Till <i7tiol@t-online.de>
+Copyright (C) 2012-2016 Olaf Till <i7tiol@t-online.de>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@ along with this program; If not, see <http://www.gnu.org/licenses/>.
 #include "pq_connection.h"
 
 // PKG_ADD: autoload ("pq_close", "pq_interface.oct");
+// PKG_DEL: autoload ("pq_close", "pq_interface.oct", "remove");
 
 DEFUN_DLD (pq_close, args, ,
            "-*- texinfo -*-\n\
@@ -40,11 +41,12 @@ Closes connection @var{connection} to a postgresql server.\n\
       return octave_value_list ();
     }
 
-  octave_base_value& rep = const_cast<octave_base_value&> (args(0).get_rep ());
+  const octave_base_value& rep = (args(0).get_rep ());
 
-  octave_pq_connection &oct_pq_conn = dynamic_cast<octave_pq_connection&> (rep);
+  const octave_pq_connection &oct_pq_conn =
+    dynamic_cast<const octave_pq_connection&> (rep);
 
-  oct_pq_conn.octave_pq_close ();
+  oct_pq_conn.get_rep ()->octave_pq_close ();
 
   return octave_value_list ();
 }
